""" Defines the base class for a Group in OpenMDAO."""

from __future__ import print_function

import sys
import os
from collections import Counter, OrderedDict
from six import iteritems, iterkeys, itervalues
from six.moves import zip_longest
from itertools import chain

import numpy as np
import networkx as nx

from openmdao.components.indep_var_comp import IndepVarComp
from openmdao.core.component import Component
from openmdao.core.mpi_wrap import MPI, debug
from openmdao.core.system import System
from openmdao.util.string_util import name_relative_to
from openmdao.util.graph import collapse_nodes
from openmdao.core.checks import ConnectError

trace = os.environ.get('OPENMDAO_TRACE')


class Group(System):
    """A system that contains other systems.

    Options
    -------
    fd_options['force_fd'] :  bool(False)
        Set to True to finite difference this system.
    fd_options['form'] :  str('forward')
        Finite difference mode. (forward, backward, central) You can also set to 'complex_step' to peform the complex step method if your components support it.
    fd_options['step_size'] :  float(1e-06)
        Default finite difference stepsize
    fd_options['step_type'] :  str('absolute')
        Set to absolute, relative

    """

    def __init__(self):
        super(Group, self).__init__()

        self._src = {}
        self._src_idxs = {}
        self._data_xfer = {}

        self._local_unknown_sizes = {}
        self._local_param_sizes = {}

        # put these in here to avoid circular imports
        from openmdao.solvers.ln_gauss_seidel import LinearGaussSeidel
        from openmdao.solvers.run_once import RunOnce
        from openmdao.solvers.scipy_gmres import ScipyGMRES

        # These solvers are the default
        self.ln_solver = ScipyGMRES()
        self.nl_solver = RunOnce()
        self.precon = LinearGaussSeidel()

        # Flag is true after order is set
        self._order_set = False

    def _subsystem(self, name):
        """
        Returns a reference to a named subsystem that is a direct or an indirect
        subsystem of the this system.  Raises an exception if the given name
        doesn't reference a subsystem.

        Args
        ----
        name : str
            Name of the subsystem to retrieve.

        Returns
        -------
        `System`
            A reference to the named subsystem.
        """
        s = self
        for part in name.split('.'):
            s = s._subsystems[part]
        return s

    def add(self, name, system, promotes=None):
        """Add a subsystem to this group, specifying its name and any variables
        that it promotes to the parent level.

        Args
        ----

        name : str
            The name by which the subsystem is to be known.

        system : `System`
            The subsystem to be added.

        promotes : tuple, optional
            The names of variables in the subsystem which are to be promoted.
        """

        # Can't call set after specifying an order
        if self._order_set:
            msg = 'You cannot call add after specifying an order.'
            raise RuntimeError(msg)

        if promotes is not None:
            system._promotes = promotes

        if name in self._subsystems:
            msg = "Group '%s' already contains a subsystem with name '%s'." % \
                  (self.name, name)
            raise RuntimeError(msg)
        elif hasattr(self, name):
            msg = "Group '%s' already contains an attribute with name '%s'." % \
                  (self.name, name)
            raise RuntimeError(msg)

        self._subsystems[name] = system
        setattr(self, name, system)
        system.name = name
        return system

    def connect(self, source, targets, src_indices=None):
        """Connect the given source variable to the given target
        variable.

        Args
        ----

        source : source
            The name of the source variable.

        targets : str OR iterable
            The name of one or more target variables.

        src_indices : index array, optional
            If specified, connect the specified entries of the full
            distributed source value to the target.
        """
        if isinstance(targets, str):
            targets = (targets,)

        for target in targets:
            self._src.setdefault(target, []).append((source, src_indices))

    def subsystems(self, local=False, recurse=False, typ=System, include_self=False):
        """
        Args
        ----
        local : bool, optional
            If True, only return those `Components` that are local. Default is False.

        recurse : bool, optional
            If True, return all `Components` in the system tree, subject to
            the value of the local arg. Default is False.

        typ : type, optional
            If a class is specified here, only those subsystems that are instances
            of that type will be returned.  Default type is `System`.

        include_self : bool, optional
            If True, yield self before iterating over subsystems, assuming type
            of self is appropriate. Default is False.

        Returns
        -------
        iterator
            Iterator over subsystems.
        """
        if include_self and isinstance(self, typ):
            yield self

        subs = self._local_subsystems if local else itervalues(self._subsystems)

        for sub in subs:
            if isinstance(sub, typ):
                yield sub
            if recurse and isinstance(sub, Group):
                for s in sub.subsystems(local, recurse, typ):
                    yield s

    def subgroups(self, local=False, recurse=False, include_self=False):
        """
        Returns
        -------
        iterator
            Iterator over subgroups.
        """
        return self.subsystems(local=local, recurse=recurse, typ=Group,
                               include_self=include_self)

    def components(self, local=False, recurse=False, include_self=False):
        """
        Returns
        -------
        iterator
            Iterator over sub-`Components`.
        """
        return self.subsystems(local=local, recurse=recurse, typ=Component,
                               include_self=include_self)

    def _init_sys_data(self, parent_path, probdata):
        """Set the absolute pathname of each `System` in the tree.

        Args
        ----
        parent_path : str
            The pathname of the parent `System`, which is to be prepended to the
            name of this child `System` and all subsystems.

        probdata : `_ProbData`
            Problem level data container.
        """
        super(Group, self)._init_sys_data(parent_path, probdata)
        for sub in itervalues(self._subsystems):
            sub._init_sys_data(self.pathname, probdata)

    def _setup_variables(self, compute_indices=False):
        """
        Create dictionaries of metadata for parameters and for unknowns for
        this `Group` and stores them as attributes of the `Group`. The
        promoted name of subsystem variables with respect to this `Group`
        is included in the metadata.

        Args
        ----
        compute_indices : bool, optional
            If True, call setup_distrib_idxs() to set values of
            'src_indices' metadata.

        Returns
        -------
        tuple
            A dictionary of metadata for parameters and for unknowns
            for all subsystems.
        """
        params_dict = OrderedDict()
        unknowns_dict = OrderedDict()

        self._params_dict = params_dict
        self._unknowns_dict = unknowns_dict

        self._sysdata._params_dict = params_dict
        self._sysdata._unknowns_dict = unknowns_dict

        self._data_xfer = {}
        self._to_abs_unames = {}
        self._to_abs_pnames = {}

        for sub in itervalues(self._subsystems):
            subparams, subunknowns = sub._setup_variables(compute_indices)
            for p, meta in iteritems(subparams):
                meta = meta.copy()
                meta['promoted_name'] = self._promoted_name(meta['promoted_name'], sub)
                params_dict[p] = meta
                self._to_abs_pnames.setdefault(meta['promoted_name'], []).append(p)

            for u, meta in iteritems(subunknowns):
                meta = meta.copy()
                meta['promoted_name'] = self._promoted_name(meta['promoted_name'], sub)
                unknowns_dict[u] = meta
                self._to_abs_unames.setdefault(meta['promoted_name'], []).append(u)

            # check for any promotes that didn't match a variable
            sub._check_promotes()

        return self._params_dict, self._unknowns_dict

    def _promoted_name(self, name, subsystem):
        """
        Returns
        -------
        str
            The promoted name of the given variable.
        """
        if subsystem._promoted(name):
            return name
        if len(subsystem.name) > 0:
            return '.'.join((subsystem.name, name))
        else:
            return name

    def _setup_communicators(self, comm):
        """
        Assign communicator to this `Group` and all of its subsystems.

        Args
        ----
        comm : an MPI communicator (real or fake)
            The communicator being offered by the parent system.
        """
        self._local_subsystems = []

        self.comm = comm

        for sub in itervalues(self._subsystems):
            sub._setup_communicators(self.comm)
            if self.is_active() and sub.is_active():
                self._local_subsystems.append(sub)

    def _setup_vectors(self, param_owners, parent=None,
                       top_unknowns=None, impl=None):
        """Create `VecWrappers` for this `Group` and all below it in the
        `System` tree.

        Args
        ----
        param_owners : dict
            A dictionary mapping `System` pathnames to the pathnames of parameters
            they are reponsible for propagating.

        parent : `Group`, optional
            The `Group` that contains this `Group`, if any.

        top_unknowns : `VecWrapper`, optional
            The `Problem` level unknowns `VecWrapper`.

        impl : an implementation factory, optional
            Specifies the factory object used to create `VecWrapper` and
            `DataTransfer` objects.
        """
        self.params = self.unknowns = self.resids = None
        self.dumat, self.dpmat, self.drmat = {}, {}, {}
        self._local_unknown_sizes = {}
        self._local_param_sizes = {}
        self._owning_ranks = None
        relevance = self._relevance

        if not self.is_active():
            return

        self._impl = impl

        my_params = param_owners.get(self.pathname, ())

        max_psize, self._shared_p_offsets = \
            self._get_shared_vec_info(self._params_dict, my_params=my_params)

        if parent is None:
            # determine the size of the largest grouping of parallel subvecs,
            # allocate an array of that size, and sub-allocate from that for all
            # relevant subvecs. We should never need more memory than the
            # largest sized collection of parallel vecs.
            max_usize, self._shared_u_offsets = \
                self._get_shared_vec_info(self._unknowns_dict)

            # other vecs will be sub-sliced from this one
            self._shared_du_vec = np.zeros(max_usize)
            self._shared_dr_vec = np.zeros(max_usize)
            self._shared_dp_vec = np.zeros(max_psize)

            self._create_vecs(my_params, voi=None, impl=impl)
            top_unknowns = self.unknowns
        else:
            self._shared_dp_vec = np.zeros(max_psize)

            # map promoted name in parent to corresponding promoted name in this view
            self._relname_map = self._get_relname_map(parent.unknowns)
            self._create_views(top_unknowns, parent, my_params, voi=None)

        self._u_size_lists = self.unknowns._get_flattened_sizes()
        self._p_size_lists = self.params._get_flattened_sizes()

        self._owning_ranks = self._get_owning_ranks()

        self._setup_data_transfer(my_params, None)

        all_vois = set([None])
        if self._probdata.top_lin_gs:
            # create storage for the relevant vecwrappers,
            # keyed by variable_of_interest
            for vois in relevance.groups:
                all_vois.update(vois)
                for voi in vois:
                    if parent is None:
                        self._create_vecs(my_params, voi, impl)
                    else:
                        self._create_views(top_unknowns, parent, my_params,
                                           voi)

                    self._setup_data_transfer(my_params, voi)

        self._setup_gs_outputs(all_vois)

        # convert any src_indices to index arrays
        for meta in itervalues(self._params_dict):
            if 'src_indices' in meta:
                meta['src_indices'] = self.params.to_idx_array(meta['src_indices'])

        for sub in itervalues(self._subsystems):
            sub._setup_vectors(param_owners, parent=self,
                               top_unknowns=top_unknowns,
                               impl=self._impl)

        # now that all of the vectors and subvecs are allocated, calculate
        # and cache the ls_inputs.
        self._ls_inputs = {}
        for voi, vec in iteritems(self.dumat):
            self._ls_inputs[voi] = self._all_params(voi)

        # cache this to speed up apply_linear
        self._abs_inputs = {}
        for voi, vec in iteritems(self.dpmat):
            self._abs_inputs[voi] = {
                meta['pathname'] for meta in itervalues(vec)
                    if not meta.get('pass_by_obj')
            }

        self._relname_map = None  # reclaim some memory

    def _create_vecs(self, my_params, voi, impl):
        """ This creates our vecs and mats. This is only called on
        the top level Group.
        """
        comm = self.comm
        sys_pathname = self.pathname
        params_dict = self._params_dict
        unknowns_dict = self._unknowns_dict

        self.comm = comm

        # create implementation specific VecWrappers
        if voi is None:
            self.unknowns = impl.create_src_vecwrapper(self._sysdata, comm)
            self.states = set((n for n, m in iteritems(self.unknowns) if m.get('state')))
            self.resids = impl.create_src_vecwrapper(self._sysdata, comm)
            self.params = impl.create_tgt_vecwrapper(self._sysdata, comm)

            # populate the VecWrappers with data
            self.unknowns.setup(unknowns_dict,
                                relevance=self._relevance,
                                var_of_interest=None, store_byobjs=True)
            self.resids.setup(unknowns_dict,
                              relevance=self._relevance,
                              var_of_interest=None)
            self.params.setup(None, params_dict, self.unknowns,
                              my_params, self.connections,
                              relevance=self._relevance,
                              var_of_interest=None, store_byobjs=True)

        if voi is None or self._probdata.top_lin_gs:
            dunknowns = impl.create_src_vecwrapper(self._sysdata, comm)
            dresids = impl.create_src_vecwrapper(self._sysdata, comm)
            dparams = impl.create_tgt_vecwrapper(self._sysdata, comm)

            dunknowns.setup(unknowns_dict, relevance=self._relevance,
                            var_of_interest=voi,
                            shared_vec=self._shared_du_vec[self._shared_u_offsets[voi]:])
            dresids.setup(unknowns_dict, relevance=self._relevance,
                          var_of_interest=voi,
                          shared_vec=self._shared_dr_vec[self._shared_u_offsets[voi]:])
            dparams.setup(None, params_dict, self.unknowns, my_params,
                          self.connections, relevance=self._relevance,
                          var_of_interest=voi,
                          shared_vec=self._shared_dp_vec[self._shared_p_offsets[voi]:])

            self.dumat[voi] = dunknowns
            self.drmat[voi] = dresids
            self.dpmat[voi] = dparams

    def _get_fd_params(self):
        """
        Get the list of parameters that are needed to perform a
        finite difference on this `Group`.

        Returns
        -------
        list of str
            List of names of params that have sources that are IndepVarComps
            or sources that are outside of this `Group` .
        """
        if self._fd_params is None:
            conns = self.connections
            mypath = self.pathname + '.' if self.pathname else ''
            mplen = len(mypath)

            params = self._fd_params = []
            for tgt, (src, idxs) in iteritems(conns):
                if mypath == tgt[:mplen]:
                    # look up the Component that contains the source variable
                    scname = src.rsplit('.', 1)[0]
                    if mypath == scname[:mplen]:
                        src_comp = self._subsystem(scname[mplen:])
                        if isinstance(src_comp, IndepVarComp):
                            params.append(tgt[mplen:])
                    else:
                        params.append(tgt[mplen:])

        return self._fd_params

    def _get_fd_unknowns(self):
        """
        Get the list of unknowns that are needed to perform a
        finite difference on this `Group`.

        Returns
        -------
        list of str
            List of names of unknowns for this `Group` that don't come from a
            `IndepVarComp`.
        """
        mypath = self.pathname + '.' if self.pathname else ''
        fd_unknowns = []
        for name, meta in iteritems(self.unknowns):
            # look up the subsystem containing the unknown
            sub = self._subsystem(meta['pathname'].rsplit('.', 1)[0][len(mypath):])
            if not isinstance(sub, IndepVarComp):
                fd_unknowns.append(name)

        return fd_unknowns

    def _get_explicit_connections(self):
        """
        Returns
        -------
        dict
            Explicit connections in this `Group`, represented as a mapping
            from the pathname of the target to the pathname of the source.
        """
        connections = {}
        for sub in self.subgroups():
            connections.update(sub._get_explicit_connections())

        for tgt, srcs in iteritems(self._src):
            for src, idxs in srcs:
                try:
                    src_pathnames = self._to_abs_unames[src]
                except KeyError as error:
                    try:
                        src_pathnames = self._to_abs_pnames[src]
                    except KeyError as error:
                        raise ConnectError.nonexistent_src_error(src, tgt)

                try:
                    for tgt_pathname in self._to_abs_pnames[tgt]:
                        for src_pathname in src_pathnames:
                            connection = (src_pathname, idxs)
                            connections.setdefault(tgt_pathname, []).append(connection)
                except KeyError as error:
                    try:
                        self._to_abs_unames[tgt]
                    except KeyError as error:
                        raise ConnectError.nonexistent_target_error(src, tgt)
                    else:
                        raise ConnectError.invalid_target_error(src, tgt)

        return connections

    def solve_nonlinear(self, params=None, unknowns=None, resids=None, metadata=None):
        """
        Solves the group using the slotted nl_solver.

        Args
        ----
        params : `VecWrapper`, optional
            `VecWrapper` containing parameters. (p)

        unknowns : `VecWrapper`, optional
            `VecWrapper` containing outputs and states. (u)

        resids : `VecWrapper`, optional
            `VecWrapper` containing residuals. (r)

        metadata : dict, optional
            Dictionary containing execution metadata (e.g. iteration coordinate).
        """
        if self.is_active():
            params = params if params is not None else self.params
            unknowns = unknowns if unknowns is not None else self.unknowns
            resids = resids if resids is not None else self.resids

            self.nl_solver.solve(params, unknowns, resids, self, metadata)

    def children_solve_nonlinear(self, metadata):
        """
        Loops over our children systems and asks them to solve.

        Args
        ----
        metadata : dict
            Dictionary containing execution metadata (e.g. iteration coordinate).
        """

        # transfer data to each subsystem and then solve_nonlinear it
        for sub in itervalues(self._subsystems):
            self._transfer_data(sub.name)
            if sub.is_active():
                if isinstance(sub, Component):
                    sub.solve_nonlinear(sub.params, sub.unknowns, sub.resids)
                else:
                    sub.solve_nonlinear(sub.params, sub.unknowns, sub.resids, metadata)

    def apply_nonlinear(self, params, unknowns, resids, metadata=None):
        """
        Evaluates the residuals of our children systems.

        Args
        ----
        params : `VecWrapper`
            `VecWrapper` containing parameters. (p)

        unknowns : `VecWrapper`
            `VecWrapper` containing outputs and states. (u)

        resids : `VecWrapper`
            `VecWrapper` containing residuals. (r)

        metadata : dict, optional
            Dictionary containing execution metadata (e.g. iteration coordinate).
        """
        if not self.is_active():
            return

        # transfer data to each subsystem and then apply_nonlinear to it
        for sub in itervalues(self._subsystems):
            self._transfer_data(sub.name)
            if sub.is_active():
                if isinstance(sub, Component):
                    sub.apply_nonlinear(sub.params, sub.unknowns, sub.resids)
                else:
                    sub.apply_nonlinear(sub.params, sub.unknowns, sub.resids, metadata)

    def linearize(self, params, unknowns, resids):
        """
        Linearize all our subsystems.

        Args
        ----
        params : `VecWrapper`
            `VecWrapper` containing parameters. (p)

        unknowns : `VecWrapper`
            `VecWrapper` containing outputs and states. (u)

        resids : `VecWrapper`
            `VecWrapper` containing residuals. (r)
        """
        for sub in self._local_subsystems:
            sub._sys_linearize(sub.params, sub.unknowns, sub.resids)

    def _sys_apply_linear(self, mode, ls_inputs=None, vois=(None,), gs_outputs=None):
        """Calls apply_linear on our children. If our child is a `Component`,
        then we need to also take care of the additional 1.0 on the diagonal
        for explicit outputs.

        df = du - dGdp * dp or du = df and dp = -dGdp^T * df

        Args
        ----

        mode : string
            Derivative mode, can be 'fwd' or 'rev'.

        ls_inputs : dict
            We can only solve derivatives for the inputs the instigating
            system has access to.

        vois: list of strings
            List of all quantities of interest to key into the mats.

        gs_outputs : dict, optional
            Linear Gauss-Siedel can limit the outputs when calling apply.
        """
        if not self.is_active():
            return

        if mode == 'fwd':
            for voi in vois:
                self._transfer_data(deriv=True, var_of_interest=voi)  # Full Scatter

        if self.fd_options['force_fd']:
            # parent class has the code to do the fd
            super(Group, self)._sys_apply_linear(mode, ls_inputs, vois, gs_outputs)

        else:
            for sub in self._local_subsystems:
                sub._sys_apply_linear(mode, ls_inputs=ls_inputs, vois=vois,
                                      gs_outputs=gs_outputs)

        if mode == 'rev':
            for voi in vois:
                self._transfer_data(mode='rev', deriv=True, var_of_interest=voi)  # Full Scatter

    def solve_linear(self, dumat, drmat, vois, mode=None, precon=False):
        """
        Single linear solution applied to whatever input is sitting in
        the rhs vector.

        Args
        ----
        dumat : dict of `VecWrappers`
            In forward mode, each `VecWrapper` contains the incoming vector
            for the states. There is one vector per quantity of interest for
            this problem. In reverse mode, it contains the outgoing vector for
            the states. (du)

        drmat : `dict of VecWrappers`
            `VecWrapper` containing either the outgoing result in forward mode
            or the incoming vector in reverse mode. There is one vector per
            quantity of interest for this problem. (dr)

        vois: list of strings
            List of all quantities of interest to key into the mats.

        mode : string, optional
            Derivative mode, can be 'fwd' or 'rev', but generally should be
            called without mode so that the user can set the mode in this
            system's ln_solver.options.

        precon : bool, optional
            Set to True to use the solver in `precon` instead of the one in
            `ln_solver`.
        """
        if not self.is_active():
            return

        if precon is True:
            solver = self.precon
        else:
            solver = self.ln_solver

        if mode is None:
            mode = self.fd_options['mode']

        if mode == 'fwd':
            sol_vec, rhs_vec = dumat, drmat
        else:
            sol_vec, rhs_vec = drmat, dumat

        # Solve Jacobian, df |-> du [fwd] or du |-> df [rev]
        rhs_buf = OrderedDict()
        for voi in vois:
            # Skip if we are all zeros.
            if rhs_vec[voi].norm() < 1e-15:
                sol_vec[voi].vec[:] = 0.0
                continue

            rhs_buf[voi] = rhs_vec[voi].vec.copy()

        if len(rhs_buf) == 0:
            return

        sol_buf = solver.solve(rhs_buf, self, mode=mode)

        for voi in rhs_buf:
            sol_vec[voi].vec[:] = sol_buf[voi][:]

    def clear_dparams(self):
        """ Zeros out the dparams (dp) vector."""

        # this was moved here from system.py because Components never own
        # their own params. All of the calls to clear_dparams on components
        # are unnecessary

        for parallel_set in self._relevance.vars_of_interest():
            for name in parallel_set:
                if name in self.dpmat:
                    self.dpmat[name].vec[:] = 0.0

        self.dpmat[None].vec[:] = 0.0

        # Recurse to clear all dparams vectors.
        for system in self._local_subsystems:
            if isinstance(system, Group):
                system.clear_dparams()  # only call on Groups

    def set_order(self, new_order):
        """ Specifies a new execution order for this system. This should only
        be called after all subsystems have been added.

        Args
        ----
        new_order : list of str
            List of system names in desired new execution order.
        """

        # Make sure the new_order is valid. It must contain all subsystems
        # in this model.
        newset = set(new_order)
        oldset = set(iterkeys(self._subsystems))
        if oldset != newset:
            missing = oldset - newset
            extra = newset - oldset

            msg = "Unexpected new order. "
            if len(missing) > 0:
                msg += "The following are missing: %s. " % list(missing)
            if len(extra) > 0:
                msg += "The following are extra: %s. " % list(extra)

            raise ValueError(msg)

        # Don't allow duplicates either.
        if len(newset) < len(new_order):
            dupes = [key for key, val in iteritems(Counter(new_order)) if val>1]
            msg = "Duplicate name(s) found in order list: %s" % dupes
            raise ValueError(msg)

        new_subs = OrderedDict()
        for sub in new_order:
            new_subs[sub] = self._subsystems[sub]

        self._subsystems = new_subs

        # reset locals
        new_locs = []
        for sub in itervalues(self._subsystems):
            if sub in self._local_subsystems:
                new_locs.append(sub)
        self._local_subsystems = new_locs

        self._order_set = True

    def list_order(self):
        """ Lists execution order for systems in this Group.

        Returns
        -------
        list of str : List of system names in execution order.
        """
        return [n for n in self._subsystems]

    def list_auto_order(self):
        """
        Returns
        -------
        list of str
            Names of subsystems listed in the order that they
            would be executed if a manual order was not set.

        list of str
            Edges that where removed from the graph to allow sorting.
        """
        graph, broken_edges = self._break_cycles(self.list_order(),
                                                 self._get_sys_graph())
        order = nx.topological_sort(graph)
        sz = len(self.pathname)+1 if self.pathname else 0
        return [n[sz:] for n in order], broken_edges

    def _get_sys_graph(self):
        """Return the subsystem graph for this Group."""

        sgraph = self._relevance._sgraph
        if self.pathname:
            path = self.pathname.split('.')
            start = self.pathname + '.'
            slen = len(start)
            graph = sgraph.subgraph((n for n in sgraph if start == n[:slen]))
        else:
            path = []
            graph = sgraph.subgraph(sgraph.nodes_iter())

        plen = len(path)+1

        renames = {}
        for node in graph.nodes_iter():
            newnode = '.'.join(node.split('.')[:plen])
            if newnode != node:
                renames[node] = newnode

        # get the graph of direct children of current group
        collapse_nodes(graph, renames, copy=False)

        return graph

    def _break_cycles(self, order, graph):
        """Keep breaking cycles until the graph is a DAG.
        """
        broken_edges = []

        strong = [s for s in nx.strongly_connected_components(graph)
                  if len(s) > 1]
        while strong:
            # First of all, see if the cycle has in edges
            in_edges = []
            start = None
            if len(strong[0]) < len(graph):
                for s in strong[0]:
                    count = len([u for u, v in graph.in_edges(s)
                                if u not in strong[0]])
                    in_edges.append((count, s))
                in_edges = sorted(in_edges)
                if in_edges[-1][0] > 0:
                    start = in_edges[-1][1]  # take the node with the most in edges

            if start is None:
                # take the first system in the existing order that is found
                # in the SCC and disconnect it from its predecessors that are
                # also found in the SCC
                for node in order:
                    if self.pathname:
                        node = '.'.join((self.pathname, node))
                    if node in strong[0]:
                        start = node
                        break

            # break cycles
            for p in graph.predecessors(start):
                if p in strong[0]:
                    graph.remove_edge(p, start)
                    broken_edges.append((p, start))

            strong = [s for s in nx.strongly_connected_components(graph)
                      if len(s) > 1]

        return graph, broken_edges

    def _all_params(self, voi=None):
        """ Returns the set of all parameters in this system and all subsystems.

        Args
        ----
        voi: string
            Variable of interest, default is None.
        """

        # TODO: clean this up
        ls_inputs = set(iterkeys(self.dpmat[voi]))
        abs_uvec = {meta['pathname'] for meta in itervalues(self.dumat[voi])}

        for comp in self.components(local=True, recurse=True):
            for intinp_rel, meta in iteritems(comp.dpmat[voi]):
                intinp_abs = meta['pathname']
                src, idxs = self.connections.get(intinp_abs)

                if src in abs_uvec:
                    ls_inputs.add(intinp_abs)

        return ls_inputs

    def dump(self, nest=0, out_stream=sys.stdout, verbose=False, dvecs=False,
             sizes=False):
        """
        Writes a formated dump of the `System` tree to file.

        Args
        ----
        nest : int, optional
            Starting nesting level.  Defaults to 0.

        out_stream : file-like, optional
            Where output is written.  Defaults to sys.stdout.

        verbose : bool, optional
            If True, output additional info beyond
            just the tree structure. Default is False.

        dvecs : bool, optional
            If True, show contents of du and dp vectors instead of
            u and p (the default).

        sizes : bool, optional
            If True, include vector sizes and comm sizes. Default is False.
        """
        klass = self.__class__.__name__
        if dvecs:
            ulabel, plabel, uvecname, pvecname = 'du', 'dp', 'dunknowns', 'dparams'
        else:
            ulabel, plabel, uvecname, pvecname = 'u', 'p', 'unknowns', 'params'

        uvec = getattr(self, uvecname)
        pvec = getattr(self, pvecname)

        template = "%s %s '%s'"
        out_stream.write(template % (" "*nest, klass, self.name))

        out_stream.write("  NL: %s  LN: %s" % (self.nl_solver.__class__.__name__,
                                               self.ln_solver.__class__.__name__))
        if sizes:
            commsz = self.comm.size if hasattr(self.comm, 'size') else 0
            template = "    req: %s  usize:%d  psize:%d  commsize:%d"
            out_stream.write(template % (self.get_req_procs(),
                                         uvec.vec.size,
                                         pvec.vec.size,
                                         commsz))
        out_stream.write("\n")

        if verbose:  # pragma: no cover
            vec_conns = dict(self._data_xfer[('', 'fwd', None)].vec_conns)
            byobj_conns = dict(self._data_xfer[('', 'fwd', None)].byobj_conns)

            # collect width info
            lens = [len(u)+sum(map(len, v)) for u, v in
                    chain(iteritems(vec_conns), iteritems(byobj_conns))]
            if lens:
                nwid = max(lens) + 9
            else:
                lens = [len(n) for n in iterkeys(uvec)]
                nwid = max(lens) if lens else 12

            for v, meta in iteritems(uvec):
                if meta.get('pass_by_obj') or meta.get('remote'):
                    continue
                out_stream.write(" "*(nest+8))
                uslice = '{0}[{1[0]}:{1[1]}]'.format(ulabel, uvec._slices[v])
                pnames = [p for p, u in iteritems(vec_conns) if u == v]

                if pnames:
                    if len(pnames) == 1:
                        pname = pnames[0]
                        pslice = pvec._slices.get(pname, (-1, -1))
                        pslice = '%d:%d' % (pslice[0], pslice[1])
                    else:
                        pslice = [('%d:%d' % pvec._slices.get(p, (-1, -1))) for p in pnames]
                        if len(pslice) > 1:
                            pslice = ','.join(pslice)
                        else:
                            pslice = pslice[0]

                    pslice = '{}[{}]'.format(plabel, pslice)

                    connstr = '%s -> %s' % (v, pnames)
                    template = "{0:<{nwid}} {1:<10} {2:<10} {3:>10}\n"
                    out_stream.write(template.format(connstr,
                                                     uslice,
                                                     pslice,
                                                     repr(uvec[v]),
                                                     nwid=nwid))
                else:
                    template = "{0:<{nwid}} {1:<21} {2:>10}\n"
                    out_stream.write(template.format(v,
                                                     uslice,
                                                     repr(uvec[v]),
                                                     nwid=nwid))

            if not dvecs:
                for dest, src in iteritems(byobj_conns):
                    out_stream.write(" "*(nest+8))
                    connstr = '%s -> %s:' % (src, dest)
                    template = "{0:<{nwid}} (by_obj)  ({1})\n"
                    out_stream.write(template.format(connstr,
                                                     repr(uvec[src]),
                                                     nwid=nwid))

        nest += 3
        for sub in self.subsystems():
            sub.dump(nest, out_stream=out_stream, verbose=verbose, dvecs=dvecs,
                     sizes=sizes)

        out_stream.flush()

    def get_req_procs(self):
        """
        Returns
        -------
        tuple
            A tuple of the form (min_procs, max_procs), indicating the min and max
            processors usable by this `Group`.
        """
        min_procs = 1
        max_procs = 1

        for sub in itervalues(self._subsystems):
            sub_min, sub_max = sub.get_req_procs()
            min_procs = max(min_procs, sub_min)
            if max_procs is not None:
                if sub_max is None:
                    max_procs = None
                else:
                    max_procs = max(max_procs, sub_max)

        return (min_procs, max_procs)

    def _get_global_idxs(self, uname, pname, top_uname, top_pname, u_var_idxs,
                         u_sizes, p_var_idxs, p_sizes, mode):
        """
        Return the global indices into the distributed unknowns and params vectors
        for the given unknown and param.  The given unknown and param have already
        been tested for relevance.

        Args
        ----
        uname : str
            Name of variable in the unknowns vector.

        pname : str
            Name of the variable in the params vector.

        u_var_idxs : OrderedDict of (name : idx)
            Names of relevant vars in the unknowns vector and their index
            into the sizes table.

        u_sizes : ndarray
            (rank x var) array of unknown sizes.

        p_var_idxs : OrderedDict of (name : idx)
            Names of relevant vars in the params vector and their index
            into the sizes table.

        p_sizes : ndarray
            (rank x var) array of parameter sizes.

        mode : str
            Solution mode, either 'fwd' or 'rev'

        Returns
        -------
        tuple of (idx_array, idx_array)
            index array into the global unknowns vector and the corresponding
            index array into the global params vector.
        """
        rev = True if mode == 'rev' else False
        umeta = self.unknowns.metadata(uname)
        pmeta = self.params.metadata(pname)

        iproc = 0 if self.comm is None else self.comm.rank
        udist = 'src_indices' in umeta
        pdist = 'src_indices' in pmeta

        # FIXME: if we switch to push scatters, this check will flip
        if ((not rev and pmeta.get('remote')) or
            (rev and not pdist and umeta.get('remote')) or
                (rev and udist and not pdist and iproc != self._owning_ranks[pname])):
            # just return empty index arrays for remote vars
            return self.params.make_idx_array(0, 0), self.params.make_idx_array(0, 0)

        if pdist:
            arg_idxs = self.params.to_idx_array(pmeta['src_indices'])
        else:
            arg_idxs = self.params.make_idx_array(0, pmeta['size'])

        ivar = u_var_idxs[uname]
        if udist or pdist:
            new_indices = np.zeros(arg_idxs.shape, dtype=arg_idxs.dtype)

            for irank in range(self.comm.size):
                start = np.sum(u_sizes[:irank, ivar])
                end = start + u_sizes[irank, ivar]
                on_irank = np.logical_and(start <= arg_idxs, arg_idxs < end)

                # Compute conversion to new ordering

                # arg_idxs are provided wrt the full distributed variable,
                # so we subtract off the start of the var in the current rank
                # in order to make the overall offset relative to the
                # beginning of the full distributed variable.
                offset = -start

                offset += np.sum(u_sizes[:irank, :])
                offset += np.sum(u_sizes[irank, :ivar])

                # Apply conversion only to relevant parts of input
                new_indices[on_irank] = arg_idxs[on_irank] + offset

            src_idxs = new_indices
            var_rank = iproc

        else:
            var_rank = self._owning_ranks[uname] if not rev else iproc
            offset = np.sum(u_sizes[:var_rank]) + np.sum(u_sizes[var_rank, :ivar])
            src_idxs = arg_idxs + offset

            var_rank = self._owning_ranks[pname] if rev else iproc

        tgt_start = (np.sum(p_sizes[:var_rank]) +
                     np.sum(p_sizes[var_rank, :p_var_idxs[pname]]))
        tgt_idxs = tgt_start + self.params.make_idx_array(0, len(arg_idxs))

        return src_idxs, tgt_idxs

    def _setup_data_transfer(self, my_params, var_of_interest):
        """
        Create `DataTransfer` objects to handle data transfer for all of the
        connections that involve parameters for which this `Group`
        is responsible.

        Args
        ----

        my_params : list
            List of pathnames for parameters that the `Group` is
            responsible for propagating.

        var_of_interest : str or None
            The name of a variable of interest.

        """
        relevance = self._relevance

        # create ordered dicts that map relevant vars to their index into
        # the sizes table.
        vec_unames = (n for n, sz in self._u_size_lists[0]
                      if relevance.is_relevant(var_of_interest,
                                               self.unknowns._sysdata._to_top_prom_name[n]))
        vec_unames = OrderedDict(((n, i) for i, n in enumerate(vec_unames)))
        vec_pnames = (n for n, sz in self._p_size_lists[0]
                      if relevance.is_relevant(var_of_interest,
                                               self.params._sysdata._to_top_prom_name[n]))
        vec_pnames = OrderedDict(((n, i) for i, n in enumerate(vec_pnames)))

        unknown_sizes = []
        param_sizes = []
        for iproc in range(self.comm.size):
            unknown_sizes.append([sz for n, sz in self._u_size_lists[iproc]
                                  if n in vec_unames])
            param_sizes.append([sz for n, sz in self._p_size_lists[iproc]
                                if n in vec_pnames])

        unknown_sizes = np.array(unknown_sizes,
                                 dtype=self._impl.idx_arr_type)
        self._local_unknown_sizes[var_of_interest] = unknown_sizes

        param_sizes = np.array(param_sizes,
                               dtype=self._impl.idx_arr_type)
        self._local_param_sizes[var_of_interest] = param_sizes

        xfer_dict = {}
        for param, (unknown, idxs) in iteritems(self.connections):
            if param in my_params:
                urelname = self.unknowns.get_promoted_varname(unknown)
                prelname = self.params.get_promoted_varname(param)

                top_urelname = self._unknowns_dict[unknown]['top_promoted_name']
                top_prelname = self._params_dict[param]['top_promoted_name']

                if not relevance.is_relevant(var_of_interest, top_urelname) or \
                   not relevance.is_relevant(var_of_interest, top_prelname):
                    continue

                umeta = self.unknowns.metadata(urelname)

                # remove our system pathname from the abs pathname of the param and
                # get the subsystem name from that

                tgt_sys = name_relative_to(self.pathname, param)
                src_sys = name_relative_to(self.pathname, unknown)

                for sname, mode in ((tgt_sys, 'fwd'), (src_sys, 'rev')):
                    src_idx_list, dest_idx_list, vec_conns, byobj_conns = \
                        xfer_dict.setdefault((sname, mode), ([], [], [], []))

                    if 'pass_by_obj' in umeta and umeta['pass_by_obj']:
                        # rev is for derivs only, so no by_obj passing needed
                        if mode == 'fwd':
                            byobj_conns.append((prelname, urelname))
                    else:  # pass by vector
                        sidxs, didxs = self._get_global_idxs(urelname, prelname,
                                                             top_urelname, top_prelname,
                                                             vec_unames, unknown_sizes,
                                                             vec_pnames, param_sizes, mode)
                        vec_conns.append((prelname, urelname))
                        src_idx_list.append(sidxs)
                        dest_idx_list.append(didxs)

        for (tgt_sys, mode), (srcs, tgts, vec_conns, byobj_conns) in iteritems(xfer_dict):
            src_idxs = self.unknowns.merge_idxs(srcs)
            tgt_idxs = self.unknowns.merge_idxs(tgts)

            if vec_conns or byobj_conns:
                self._data_xfer[(tgt_sys, mode, var_of_interest)] = \
                    self._impl.create_data_xfer(self.dumat[var_of_interest],
                                                self.dpmat[var_of_interest],
                                                src_idxs, tgt_idxs,
                                                vec_conns, byobj_conns,
                                                mode)

        # create a DataTransfer object that combines all of the
        # individual subsystem src_idxs, tgt_idxs, and byobj_conns, so that a 'full'
        # scatter to all subsystems can be done at the same time.  Store that DataTransfer
        # object under the name ''.

        for mode in ('fwd', 'rev'):
            full_srcs = []
            full_tgts = []
            full_flats = []
            full_byobjs = []
            for (tgt_sys, direction), (srcs, tgts, flats, byobjs) in iteritems(xfer_dict):
                if mode == direction:
                    full_srcs.extend(srcs)
                    full_tgts.extend(tgts)
                    full_flats.extend(flats)
                    full_byobjs.extend(byobjs)

            src_idxs = self.unknowns.merge_idxs(full_srcs)
            tgt_idxs = self.unknowns.merge_idxs(full_tgts)
            self._data_xfer[('', mode, var_of_interest)] = \
                self._impl.create_data_xfer(self.dumat[var_of_interest],
                                            self.dpmat[var_of_interest],
                                            src_idxs, tgt_idxs,
                                            full_flats, full_byobjs,
                                            mode)

    def _transfer_data(self, target_sys='', mode='fwd', deriv=False,
                       var_of_interest=None):
        """
        Transfer data to/from target_system depending on mode.

        Args
        ----

        target_sys : str, optional
            Name of the target `System`.  A name of '', the default, indicates that data
            should be transfered to all subsystems at once.

        mode : { 'fwd', 'rev' }, optional
            Specifies forward or reverse data transfer. Default is 'fwd'.

        deriv : bool, optional
            If True, use du/dp for scatter instead of u/p.  Default is False.

        var_of_interest : str or None
            Specifies the variable of interest to determine relevance.

        """
        x = self._data_xfer.get((target_sys, mode, var_of_interest))
        if x is not None:
            if deriv:
                x.transfer(self.dumat[var_of_interest], self.dpmat[var_of_interest],
                           mode, deriv=True)
            else:
                x.transfer(self.unknowns, self.params, mode)

    def _get_owning_ranks(self):
        """
        Determine the 'owning' rank of each variable and return a dict
        mapping variables to their owning rank. The owning rank is the lowest
        rank where the variable is local.

        """
        ranks = {}

        local_vars = [k for k, m in iteritems(self.unknowns) if not m.get('remote')]
        local_vars.extend([k for k, m in iteritems(self.params) if not m.get('remote')])

        if MPI:
            if trace:  # pragma: no cover
                debug("allgathering local varnames: locals = ", local_vars)
            all_locals = self.comm.allgather(local_vars)
        else:
            all_locals = [local_vars]

        for rank, vnames in enumerate(all_locals):
            for v in vnames:
                if v not in ranks:
                    ranks[v] = rank

        return ranks

    def _get_relname_map(self, unknowns):
        """
        Args
        ----
        unknowns : `VecWrapper`
            A dict-like object containing variables keyed using promoted names.

        Returns
        -------
        dict
            Maps promoted name in parent (owner of unknowns and unknowns_dict) to
            the corresponding promoted name in the child.
        """
        # unknowns is keyed on promoted name relative to the parent system
        # unknowns_dict is keyed on absolute pathname

        # use an ordered dict here so we can use this smaller dict to loop over in get_view
        umap = OrderedDict()

        for abspath, meta in iteritems(self._unknowns_dict):
            umap[unknowns.get_promoted_varname(abspath)] = meta['promoted_name']

        return umap

    def _dump_dist_idxs(self, stream=sys.stdout, recurse=True):  # pragma: no cover
        """For debugging.  prints out the distributed idxs along with the
        variables they correspond to for the u and p vectors, for example:

        C3.y     26
        C3.y     25
        C3.y     24
        C2.y     23
        C2.y     22
        C2.y     21
        sub.C3.y 20     20 C3.x
        sub.C3.y 19     19 C3.x
        sub.C3.y 18     18 C3.x
        C1.y     17     17 C2.x
        P.x      16     16 C2.x
        P.x      15     15 C2.x
        P.x      14     14 sub.C3.x
        C3.y     13     13 sub.C3.x
        C3.y     12     12 sub.C3.x
        C3.y     11     11 C1.x
        C2.y     10     10 C3.x
        C2.y      9      9 C3.x
        C2.y      8      8 C3.x
        sub.C2.y  7      7 C2.x
        sub.C2.y  6      6 C2.x
        sub.C2.y  5      5 C2.x
        C1.y      4      4 sub.C2.x
        C1.y      3      3 sub.C2.x
        P.x       2      2 sub.C2.x
        P.x       1      1 C1.x
        P.x       0      0 C1.x
        """

        def _dump(g, stream=sys.stdout):
            stream.write("\nDistributed u and p vecs for system '%s'\n\n" % g.pathname)
            idx = 0
            pdata = []
            pnwid = 0
            piwid = 0
            for lst in g._p_size_lists:
                for name, sz in lst:
                    for i in range(sz):
                        pdata.append((name, str(idx)))
                        pnwid = max(pnwid, len(name))
                        piwid = max(piwid, len(pdata[-1][1]))
                        idx += 1
                # insert a blank line to visually sparate processes
                pdata.append(('', '', '', ''))

            idx = 0
            udata = []
            unwid = 0
            uiwid = 0
            for lst in g._u_size_lists:
                for name, sz in lst:
                    for i in range(sz):
                        udata.append((name, str(idx)))
                        unwid = max(unwid, len(name))
                        uiwid = max(uiwid, len(udata[-1][1]))
                        idx += 1
                # insert a blank line to visually sparate processes
                udata.append(('', '', '', ''))

            data = []
            for u, p in zip_longest(udata, pdata, fillvalue=('', '')):
                data.append((u[0], u[1], p[1], p[0]))

            for d in data[::-1]:
                template = "{0:<{wid0}} {1:>{wid1}}     {2:>{wid2}} {3:<{wid3}}\n"
                stream.write(template.format(d[0], d[1], d[2], d[3],
                                             wid0=unwid, wid1=uiwid,
                                             wid2=piwid, wid3=pnwid))
            stream.write("\n\n")

        if recurse:
            for s in self.subgroups(recurse=True, include_self=True):
                if s.is_active():
                    _dump(s, stream)
        else:
            _dump(self, stream)



"""functions useful for debugging openmdao"""
from __future__ import print_function

import sys
from pprint import pformat
from functools import wraps
from resource import getrusage, RUSAGE_SELF, RUSAGE_CHILDREN


def dump_meta(system, nest=0, out_stream=sys.stdout):
    """
    Dumps the system tree with associated metadata for the params and unknowns
    `VecWrappers`.

    Args
    ----
    system : `System`
        The node in the `System` tree where dumping begins.

    nest : int, optional
        Starting nesting level.  Defaults to 0.

    out_stream : file-like, optional
        Where output is written.  Defaults to sys.stdout.

    """
    klass = system.__class__.__name__

    commsz = system.comm.size if hasattr(system.comm, 'size') else 0

    margin = ' '*nest
    if system.is_active():
        out_stream.write("%s %s '%s'    req: %s  usize:%d  psize:%d  commsize:%d\n" %
                         (margin,
                          klass,
                          system.name,
                          system.get_req_procs(),
                          system.unknowns.vec.size,
                          system.params.vec.size,
                          commsz))

        margin = ' '*(nest+6)
        out_stream.write("%sunknowns:\n" % margin)
        for v, meta in system.unknowns.items():
            out_stream.write("%s%s: " % (margin, v))
            out_stream.write(pformat(meta, indent=nest+9).replace("{","{\n",1))
            out_stream.write('\n')

        out_stream.write("%sparams:\n" % margin)
        for v, meta in system.params.items():
            out_stream.write("%s%s: " % (margin, v))
            out_stream.write(pformat(meta, indent=nest+9).replace("{","{\n",1))
            out_stream.write('\n')
    else:
        out_stream.write("%s %s '%s'   (inactive)\n" %
                         (margin, klass, system.name))

    nest += 3
    for sub in itervalues(system._subsystems):
        sub.dump_meta(nest, out_stream=out_stream)

    out_stream.flush()

def max_mem_usage():
    """
    Returns
    -------
    The max memory used by this process and its children, in MB.
    """
    denom = 1024.
    if sys.platform == 'darwin':
        denom *= denom
    total = getrusage(RUSAGE_SELF).ru_maxrss / denom
    total += getrusage(RUSAGE_CHILDREN).ru_maxrss / denom
    return total


def diff_max_mem(fn):
    """
    This gives the difference in max memory before and after the
    decorated function is called.  Results can sometimes be
    deceptive since it only deals with max memory, i.e., the
    value coming back from getrusage never goes down, even if
    memory is freed up.
    """
    @wraps(fn)
    def wrapper(*args, **kwargs):
        startmem = max_mem_usage()
        ret = fn(*args, **kwargs)
        diff = max_mem_usage()-startmem
        if diff > 0.0:
            print("%s added %s MB" % (fn.__name__, diff))
        return ret
    return wrapper



""" Testing optimizer ScipyOptimize."""

import os

import unittest

import numpy as np

from openmdao.api import Problem, Group, IndepVarComp, ExecComp, ScipyOptimizer, \
    LinearGaussSeidel
from openmdao.test.sellar import SellarStateConnection
from openmdao.test.util import assert_rel_error


# check that pyoptsparse is installed
# if it is, try to use SNOPT but fall back to SLSQP
OPT = None
OPTIMIZER = None

try:
    from pyoptsparse import OPT
    try:
        OPT('SNOPT')
        OPTIMIZER = 'SNOPT'
    except:
        try:
            OPT('SLSQP')
            OPTIMIZER = 'SLSQP'
        except:
            pass
except:
    pass

if OPTIMIZER:
    from openmdao.drivers.pyoptsparse_driver import pyOptSparseDriver


class TestParamIndicesScipy(unittest.TestCase):

    def test_Sellar_state_SLSQP(self):
        """ Baseline Sellar test case without specifying indices.
        """

        prob = Problem()
        prob.root = SellarStateConnection()

        prob.driver = ScipyOptimizer()
        prob.driver.options['optimizer'] = 'SLSQP'
        prob.driver.options['tol'] = 1.0e-8

        prob.driver.add_desvar('z', low=np.array([-10.0, 0.0]),
                                    high=np.array([10.0, 10.0]))
        prob.driver.add_desvar('x', low=0.0, high=10.0)

        prob.driver.add_objective('obj')
        prob.driver.add_constraint('con1', upper=0.0)
        prob.driver.add_constraint('con2', upper=0.0)
        prob.driver.options['disp'] = False

        prob.setup(check=False)
        prob.run()

        assert_rel_error(self, prob['z'][0], 1.9776, 1e-3)
        assert_rel_error(self, prob['z'][1], 0.0, 1e-3)
        assert_rel_error(self, prob['x'], 0.0, 1e-3)

    def test_driver_param_indices_slsqp(self):
        """ Test driver param indices with ScipyOptimizer SLSQP and force_fd=False
        """

        prob = Problem()
        prob.root = SellarStateConnection()

        prob.driver = ScipyOptimizer()
        prob.driver.options['optimizer'] = 'SLSQP'
        prob.driver.options['tol'] = 1.0e-8
        prob.root.fd_options['force_fd'] = False

        prob.driver.add_desvar('z', low=np.array([-10.0]),
                                    high=np.array([10.0]), indices=[0])
        prob.driver.add_desvar('x', low=0.0, high=10.0)

        prob.driver.add_objective('obj')
        prob.driver.add_constraint('con1', upper=0.0)
        prob.driver.add_constraint('con2', upper=0.0)
        #prob.driver.options['disp'] = False

        prob.setup(check=False)

        prob['z'][1] = 0.0

        prob.run()

        assert_rel_error(self, prob['z'][0], 1.9776, 1e-3)
        assert_rel_error(self, prob['z'][1], 0.0, 1e-3)
        assert_rel_error(self, prob['x'], 0.0, 1e-3)

    def test_driver_param_indices_slsqp_force_fd(self):
        """ Test driver param indices with ScipyOptimizer SLSQP and force_fd=True
        """

        prob = Problem()
        prob.root = SellarStateConnection()
        prob.root.fd_options['force_fd'] = True

        prob.driver = ScipyOptimizer()
        prob.driver.options['optimizer'] = 'SLSQP'
        prob.driver.options['tol'] = 1.0e-8

        prob.driver.add_desvar('z', low=np.array([-10.0]),
                                    high=np.array([10.0]), indices=[0])
        prob.driver.add_desvar('x', low=0.0, high=10.0)

        prob.driver.add_objective('obj')
        prob.driver.add_constraint('con1', upper=0.0)
        prob.driver.add_constraint('con2', upper=0.0)
        #prob.driver.options['disp'] = False

        prob.setup(check=False)

        prob['z'][1] = 0.0

        prob.run()

        assert_rel_error(self, prob['z'][0], 1.9776, 1e-3)
        assert_rel_error(self, prob['z'][1], 0.0, 1e-3)
        assert_rel_error(self, prob['x'], 0.0, 1e-3)


class TestParamIndicesPyoptsparse(unittest.TestCase):

    def setUp(self):
        if OPT is None:
            raise unittest.SkipTest("pyoptsparse is not installed")

        if OPTIMIZER is None:
            raise unittest.SkipTest("pyoptsparse is not providing SNOPT or SLSQP")

    def tearDown(self):
        try:
            os.remove('SLSQP.out')
        except OSError:
            pass

        try:
            os.remove('SNOPT_print.out')
            os.remove('SNOPT_summary.out')
        except OSError:
            pass

    def test_driver_param_indices(self):
        """ Test driver param indices with pyOptSparse and force_fd=False
        """

        prob = Problem()
        prob.root = SellarStateConnection()
        prob.root.fd_options['force_fd'] = False

        prob.driver = pyOptSparseDriver()
        prob.driver.options['optimizer'] = OPTIMIZER

        prob.driver.add_desvar('z', low=np.array([-10.0]),
                                    high=np.array([10.0]), indices=[0])
        prob.driver.add_desvar('x', low=0.0, high=10.0)

        prob.driver.add_objective('obj')
        prob.driver.add_constraint('con1', upper=0.0)
        prob.driver.add_constraint('con2', upper=0.0)

        prob.setup(check=False)

        prob['z'][1] = 0.0

        prob.run()

        assert_rel_error(self, prob['z'][0], 1.9776, 1e-3)
        assert_rel_error(self, prob['z'][1], 0.0, 1e-3)
        assert_rel_error(self, prob['x'], 0.0, 1e-3)

    def test_driver_param_indices_force_fd(self):
        """ Test driver param indices with pyOptSparse and force_fd=True
        """

        prob = Problem()
        prob.root = SellarStateConnection()
        prob.root.fd_options['force_fd'] = True

        prob.driver = pyOptSparseDriver()
        prob.driver.options['optimizer'] = OPTIMIZER

        prob.driver.add_desvar('z', low=np.array([-10.0]),
                                    high=np.array([10.0]), indices=[0])
        prob.driver.add_desvar('x', low=0.0, high=10.0)

        prob.driver.add_objective('obj')
        prob.driver.add_constraint('con1', upper=0.0)
        prob.driver.add_constraint('con2', upper=0.0)
        #prob.driver.options['disp'] = False

        prob.setup(check=False)

        prob['z'][1] = 0.0

        prob.run()

        assert_rel_error(self, prob['z'][0], 1.9776, 1e-3)
        assert_rel_error(self, prob['z'][1], 0.0, 1e-3)
        assert_rel_error(self, prob['x'], 0.0, 1e-3)

    def test_driver_param_indices_force_fd_shift(self):
        """ Test driver param indices with shifted indices and force_fd=True
        """

        prob = Problem()
        prob.root = SellarStateConnection()
        prob.root.fd_options['force_fd'] = True

        prob.driver.add_desvar('z', low=np.array([-10.0, -10.0]),
                                    high=np.array([10.0, 10.0]), indices=[1])
        prob.driver.add_desvar('x', low=0.0, high=10.0)

        prob.driver.add_objective('obj')
        prob.driver.add_constraint('con1', upper=0.0)
        prob.driver.add_constraint('con2', upper=0.0)
        #prob.driver.options['disp'] = False

        prob.setup(check=False)

        prob['z'][1] = 0.0

        prob.run()

        J = prob.calc_gradient(['x', 'z'], ['obj'], mode='fd',
                               return_format='array')
        assert_rel_error(self, J[0][1], 1.78402, 1e-3)

    def test_poi_index_w_irrelevant_var(self):
        prob = Problem()
        prob.driver = pyOptSparseDriver()
        prob.driver.options['optimizer'] = OPTIMIZER
        prob.root = root = Group()
        prob.root.ln_solver = LinearGaussSeidel()
        prob.root.ln_solver.options['single_voi_relevance_reduction'] = True

        root.add('p1', IndepVarComp('x', np.array([1.0, 3.0, 4.0])))
        root.add('p2', IndepVarComp('x', np.array([5.0, 2.0, -1.0])))
        root.add('C1', ExecComp('y = 2.0*x', x=np.zeros(3), y=np.zeros(3)))
        root.add('C2', ExecComp('y = 3.0*x', x=np.zeros(3), y=np.zeros(3)))
        root.add('con1', ExecComp('c = 7.0 - y', y=np.zeros(3), c=np.zeros(3)))
        root.add('con2', ExecComp('c = 2.0 - y', y=np.zeros(3), c=np.zeros(3)))
        root.add('obj', ExecComp('o = y1+y2'))

        prob.driver.add_desvar('p1.x', indices=[1])
        prob.driver.add_desvar('p2.x', indices=[2])
        prob.driver.add_constraint('con1.c', upper=0.0, indices=[1])
        prob.driver.add_constraint('con2.c', upper=0.0, indices=[2])
        prob.driver.add_objective('obj.o')

        root.connect('p1.x', 'C1.x')
        root.connect('p2.x', 'C2.x')
        root.connect('C1.y', 'con1.y')
        root.connect('C2.y', 'con2.y')
        root.connect('C1.y', 'obj.y1', src_indices=[1])
        root.connect('C2.y', 'obj.y2', src_indices=[2])

        prob.root.ln_solver.options['mode'] = 'rev'
        prob.setup(check=False)
        prob.run()

        # I was trying in this test to duplicate an error in pointer, but wasn't able to.
        # I was able to find a different error that occurred when using return_format='array'
        # that was also fixed by the same PR that fixed pointer.
        J = prob.calc_gradient(['p1.x', 'p2.x'], ['con1.c', 'con2.c'], mode='rev',
                               return_format='array')

        assert_rel_error(self, J[0][0], -2.0, 1e-3)
        assert_rel_error(self, J[0][1], .0, 1e-3)
        assert_rel_error(self, J[1][0], .0, 1e-3)
        assert_rel_error(self, J[1][1], -3.0, 1e-3)

        J = prob.calc_gradient(['p1.x', 'p2.x'], ['con1.c', 'con2.c'], mode='rev',
                               return_format='dict')

        assert_rel_error(self, J['con1.c']['p1.x'], -2.0, 1e-3)
        assert_rel_error(self, J['con1.c']['p2.x'], .0, 1e-3)
        assert_rel_error(self, J['con2.c']['p1.x'], .0, 1e-3)
        assert_rel_error(self, J['con2.c']['p2.x'], -3.0, 1e-3)

        prob.root.ln_solver.options['mode'] = 'fwd'
        prob.setup(check=False)
        prob.run()

        J = prob.calc_gradient(['p1.x', 'p2.x'], ['con1.c', 'con2.c'], mode='fwd',
                               return_format='array')

        assert_rel_error(self, J[0][0], -2.0, 1e-3)
        assert_rel_error(self, J[0][1], .0, 1e-3)
        assert_rel_error(self, J[1][0], .0, 1e-3)
        assert_rel_error(self, J[1][1], -3.0, 1e-3)

        J = prob.calc_gradient(['p1.x', 'p2.x'], ['con1.c', 'con2.c'], mode='fwd',
                               return_format='dict')

        assert_rel_error(self, J['con1.c']['p1.x'], -2.0, 1e-3)
        assert_rel_error(self, J['con1.c']['p2.x'], .0, 1e-3)
        assert_rel_error(self, J['con2.c']['p1.x'], .0, 1e-3)
        assert_rel_error(self, J['con2.c']['p2.x'], -3.0, 1e-3)


if __name__ == "__main__":
    unittest.main()



""" Unit test for the Nonlinear Gauss Seidel nonlinear solver. """

import unittest

from openmdao.api import Problem, NLGaussSeidel
from openmdao.test.sellar import SellarNoDerivatives, SellarDerivativesGrouped
from openmdao.test.util import assert_rel_error


class TestNLGaussSeidel(unittest.TestCase):

    def test_sellar(self):

        prob = Problem()
        prob.root = SellarNoDerivatives()
        prob.root.nl_solver = NLGaussSeidel()

        prob.setup(check=False)
        prob.run()

        assert_rel_error(self, prob['y1'], 25.58830273, .00001)
        assert_rel_error(self, prob['y2'], 12.05848819, .00001)

        # Make sure we aren't iterating like crazy
        self.assertLess(prob.root.nl_solver.iter_count, 8)

    def test_sellar_group(self):

        prob = Problem()
        prob.root = SellarDerivativesGrouped()
        prob.root.nl_solver = NLGaussSeidel()
        prob.root.nl_solver.options['atol'] = 1e-9
        prob.root.mda.nl_solver.options['atol'] = 1e-3
        prob.root.nl_solver.options['iprint'] = 1 # so that print_norm is in coverage

        prob.setup(check=False)
        prob.run()

        assert_rel_error(self, prob['y1'], 25.58830273, .00001)
        assert_rel_error(self, prob['y2'], 12.05848819, .00001)


if __name__ == "__main__":
    unittest.main()



""" Test objects for the sellar two discipline problem.
From Sellar's analytic problem.

    Sellar, R. S., Batill, S. M., and Renaud, J. E., "Response Surface Based, Concurrent Subspace
    Optimization for Multidisciplinary System Design," Proceedings References 79 of the 34th AIAA
    Aerospace Sciences Meeting and Exhibit, Reno, NV, January 1996.
"""

import numpy as np

from openmdao.components.exec_comp import ExecComp
from openmdao.components.indep_var_comp import IndepVarComp
from openmdao.core.component import Component
from openmdao.core.group import Group
from openmdao.solvers.nl_gauss_seidel import NLGaussSeidel
from openmdao.solvers.newton import Newton


class SellarDis1(Component):
    """Component containing Discipline 1 -- no derivatives version."""

    def __init__(self):
        super(SellarDis1, self).__init__()

        # Global Design Variable
        self.add_param('z', val=np.zeros(2))

        # Local Design Variable
        self.add_param('x', val=0.)

        # Coupling parameter
        self.add_param('y2', val=1.0)

        # Coupling output
        self.add_output('y1', val=1.0)

    def solve_nonlinear(self, params, unknowns, resids):
        """Evaluates the equation
        y1 = z1**2 + z2 + x1 - 0.2*y2"""

        z1 = params['z'][0]
        z2 = params['z'][1]
        x1 = params['x']
        y2 = params['y2']

        unknowns['y1'] = z1**2 + z2 + x1 - 0.2*y2


class SellarDis1withDerivatives(SellarDis1):
    """Component containing Discipline 1 -- derivatives version."""

    def linearize(self, params, unknowns, resids):
        """ Jacobian for Sellar discipline 1."""
        J = {}

        J['y1','y2'] = -0.2
        J['y1','z'] = np.array([[2.0*params['z'][0], 1.0]])
        J['y1','x'] = 1.0

        return J


class SellarDis2(Component):
    """Component containing Discipline 2 -- no derivatives version."""

    def __init__(self):
        super(SellarDis2, self).__init__()

        # Global Design Variable
        self.add_param('z', val=np.zeros(2))

        # Coupling parameter
        self.add_param('y1', val=1.0)

        # Coupling output
        self.add_output('y2', val=1.0)

    def solve_nonlinear(self, params, unknowns, resids):
        """Evaluates the equation
        y2 = y1**(.5) + z1 + z2"""

        z1 = params['z'][0]
        z2 = params['z'][1]
        y1 = params['y1']

        # Note: this may cause some issues. However, y1 is constrained to be
        # above 3.16, so lets just let it converge, and the optimizer will
        # throw it out
        y1 = abs(y1)

        unknowns['y2'] = y1**.5 + z1 + z2


class SellarDis2withDerivatives(SellarDis2):
    """Component containing Discipline 2 -- derivatives version."""

    def linearize(self, params, unknowns, resids):
        """ Jacobian for Sellar discipline 2."""
        J = {}

        J['y2', 'y1'] = .5*params['y1']**-.5
        J['y2', 'z'] = np.array([[1.0, 1.0]])

        return J


class SellarNoDerivatives(Group):
    """ Group containing the Sellar MDA. This version uses the disciplines
    without derivatives."""

    def __init__(self):
        super(SellarNoDerivatives, self).__init__()

        self.add('px', IndepVarComp('x', 1.0), promotes=['*'])
        self.add('pz', IndepVarComp('z', np.array([5.0, 2.0])), promotes=['*'])

        self.add('d1', SellarDis1(), promotes=['*'])
        self.add('d2', SellarDis2(), promotes=['*'])

        self.add('obj_cmp', ExecComp('obj = x**2 + z[1] + y1 + exp(-y2)',
                                     z=np.array([0.0, 0.0]), x=0.0),
                 promotes=['*'])

        self.add('con_cmp1', ExecComp('con1 = 3.16 - y1'), promotes=['*'])
        self.add('con_cmp2', ExecComp('con2 = y2 - 24.0'), promotes=['*'])

        self.nl_solver = NLGaussSeidel()
        self.d1.fd_options['force_fd'] = True
        self.d2.fd_options['force_fd'] = True


class SellarDerivatives(Group):
    """ Group containing the Sellar MDA. This version uses the disciplines
    with derivatives."""

    def __init__(self):
        super(SellarDerivatives, self).__init__()

        self.add('px', IndepVarComp('x', 1.0), promotes=['*'])
        self.add('pz', IndepVarComp('z', np.array([5.0, 2.0])), promotes=['*'])

        self.add('d1', SellarDis1withDerivatives(), promotes=['*'])
        self.add('d2', SellarDis2withDerivatives(), promotes=['*'])

        self.add('obj_cmp', ExecComp('obj = x**2 + z[1] + y1 + exp(-y2)',
                                     z=np.array([0.0, 0.0]), x=0.0),
                 promotes=['*'])

        self.add('con_cmp1', ExecComp('con1 = 3.16 - y1'), promotes=['*'])
        self.add('con_cmp2', ExecComp('con2 = y2 - 24.0'), promotes=['*'])

        self.nl_solver = NLGaussSeidel()


class SellarDerivativesGrouped(Group):
    """ Group containing the Sellar MDA. This version uses the disciplines
    without derivatives."""

    def __init__(self):
        super(SellarDerivativesGrouped, self).__init__()

        self.add('px', IndepVarComp('x', 1.0), promotes=['*'])
        self.add('pz', IndepVarComp('z', np.array([5.0, 2.0])), promotes=['*'])
        sub = self.add('mda', Group(), promotes=['*'])

        sub.add('d1', SellarDis1withDerivatives(), promotes=['*'])
        sub.add('d2', SellarDis2withDerivatives(), promotes=['*'])

        self.add('obj_cmp', ExecComp('obj = x**2 + z[1] + y1 + exp(-y2)',
                                     z=np.array([0.0, 0.0]), x=0.0, y1=0.0, y2=0.0),
                 promotes=['*'])

        self.add('con_cmp1', ExecComp('con1 = 3.16 - y1'), promotes=['*'])
        self.add('con_cmp2', ExecComp('con2 = y2 - 24.0'), promotes=['*'])

        sub.nl_solver = NLGaussSeidel()
        sub.d1.fd_options['force_fd'] = True
        sub.d2.fd_options['force_fd'] = True


class StateConnection(Component):
    """ Define connection with an explicit equation"""

    def __init__(self):
        super(StateConnection, self).__init__()

        # Inputs
        self.add_param('y2_actual', 1.0)

        # States
        self.add_state('y2_command', val=1.0)

    def apply_nonlinear(self, params, unknowns, resids):
        """ Don't solve; just calculate the residual."""

        y2_actual = params['y2_actual']
        y2_command = unknowns['y2_command']

        resids['y2_command'] = y2_actual - y2_command

    def solve_nonlinear(self, params, unknowns, resids):
        """ This is a dummy comp that doesn't modify its state."""
        pass

    def linearize(self, params, unknowns, resids):
        """Analytical derivatives."""

        J = {}

        # State equation
        J[('y2_command', 'y2_command')] = -1.0
        J[('y2_command', 'y2_actual')] = 1.0

        return J

class SellarStateConnection(Group):
    """ Group containing the Sellar MDA. This version uses the disciplines
    with derivatives."""

    def __init__(self):
        super(SellarStateConnection, self).__init__()

        self.add('px', IndepVarComp('x', 1.0), promotes=['*'])
        self.add('pz', IndepVarComp('z', np.array([5.0, 2.0])), promotes=['*'])

        self.add('state_eq', StateConnection())
        self.add('d1', SellarDis1withDerivatives(), promotes=['x', 'z', 'y1'])
        self.add('d2', SellarDis2withDerivatives(), promotes=['z', 'y1'])

        self.connect('state_eq.y2_command', 'd1.y2')
        self.connect('d2.y2', 'state_eq.y2_actual')

        self.add('obj_cmp', ExecComp('obj = x**2 + z[1] + y1 + exp(-y2)',
                                     z=np.array([0.0, 0.0]), x=0.0, y1=0.0, y2=0.0),
                 promotes=['x', 'z', 'y1', 'obj'])
        self.connect('d2.y2', 'obj_cmp.y2')

        self.add('con_cmp1', ExecComp('con1 = 3.16 - y1'), promotes=['*'])
        self.add('con_cmp2', ExecComp('con2 = y2 - 24.0'), promotes=['con2'])
        self.connect('d2.y2', 'con_cmp2.y2')

        self.nl_solver = Newton()



""" Non-linear solver that implements a Newton's method."""

from math import isnan

from openmdao.solvers.solver_base import NonLinearSolver
from openmdao.util.record_util import update_local_meta, create_local_meta


class Newton(NonLinearSolver):
    """A python Newton solver with line-search adapation of the relaxation
    parameter.
    
    Options
    -------
    options['alpha'] :  float(1.0)
        Initial over-relaxation factor.
    options['atol'] :  float(1e-12)
        Absolute convergence tolerance.
    options['iprint'] :  int(0)
        Set to 0 to disable printing, set to 1 to print the residual to stdout each iteration, set to 2 to print subiteration residuals as well.
    options['ls_atol'] :  float(1e-10)
        Absolute convergence tolerancee for line search.
    options['ls_maxiter'] :  int(10)
        Maximum number of line searches.
    options['ls_rtol'] :  float(0.9)
        Relative convergence tolerancee for line search.
    options['maxiter'] :  int(20)
        Maximum number of iterations.
    options['rtol'] :  float(1e-10)
        Relative convergence tolerance.
    options['solve_subsystems'] :  bool(True)
        Set to True to solve subsystems. You may need this for solvers nested under Newton.

    """

    def __init__(self):
        super(Newton, self).__init__()

        opt = self.options
        opt.add_option('atol', 1e-12,
                       desc='Absolute convergence tolerance.')
        opt.add_option('rtol', 1e-10,
                       desc='Relative convergence tolerance.')
        opt.add_option('maxiter', 20,
                       desc='Maximum number of iterations.')
        opt.add_option('ls_atol', 1e-10,
                       desc='Absolute convergence tolerancee for line search.')
        opt.add_option('ls_rtol', 0.9,
                       desc='Relative convergence tolerancee for line search.')
        opt.add_option('ls_maxiter', 10,
                       desc='Maximum number of line searches.')
        opt.add_option('alpha', 1.0,
                       desc='Initial over-relaxation factor.')
        opt.add_option('solve_subsystems', True,
                       desc='Set to True to solve subsystems. You may need this for solvers nested under Newton.')

    def solve(self, params, unknowns, resids, system, metadata=None):
        """ Solves the system using a Netwon's Method.

        Args
        ----
        params : `VecWrapper`
            `VecWrapper` containing parameters. (p)

        unknowns : `VecWrapper`
            `VecWrapper` containing outputs and states. (u)

        resids : `VecWrapper`
            `VecWrapper` containing residuals. (r)

        system : `System`
            Parent `System` object.

        metadata : dict, optional
            Dictionary containing execution metadata (e.g. iteration coordinate).
        """

        atol = self.options['atol']
        rtol = self.options['rtol']
        maxiter = self.options['maxiter']
        ls_atol = self.options['ls_atol']
        ls_rtol = self.options['ls_rtol']
        ls_maxiter = self.options['ls_maxiter']
        alpha = self.options['alpha']

        # Metadata setup
        self.iter_count = 0
        ls_itercount = 0
        local_meta = create_local_meta(metadata, system.pathname)
        system.ln_solver.local_meta = local_meta
        update_local_meta(local_meta, (self.iter_count, ls_itercount))

        # Perform an initial run to propagate srcs to targets.
        system.children_solve_nonlinear(local_meta)
        system.apply_nonlinear(params, unknowns, resids)

        f_norm = resids.norm()
        f_norm0 = f_norm

        if self.options['iprint'] > 0:
            self.print_norm('NEWTON', system.pathname, 0, f_norm, f_norm0)

        arg = system.drmat[None]
        result = system.dumat[None]

        alpha_base = alpha
        while self.iter_count < maxiter and f_norm > atol and \
                f_norm/f_norm0 > rtol:

            # Linearize Model with partial derivatives
            system._sys_linearize(params, unknowns, resids, total_derivs=False)

            # Calculate direction to take step
            arg.vec[:] = resids.vec
            system.solve_linear(system.dumat, system.drmat, [None], mode='fwd')

            unknowns.vec[:] += alpha*result.vec

            # Metadata update
            self.iter_count += 1
            ls_itercount = 0
            update_local_meta(local_meta, (self.iter_count, ls_itercount))

            # Just evaluate the model with the new points
            if self.options['solve_subsystems'] is True:
                system.children_solve_nonlinear(local_meta)
            system.apply_nonlinear(params, unknowns, resids, local_meta)

            self.recorders.record_iteration(system, local_meta)

            f_norm = resids.norm()
            if self.options['iprint'] > 0:
                self.print_norm('NEWTON', system.pathname, self.iter_count, f_norm, f_norm0)

            # Backtracking Line Search
            while ls_itercount < ls_maxiter and \
                    f_norm > ls_atol and \
                    f_norm/f_norm0 > ls_rtol:

                alpha *= 0.5
                unknowns.vec[:] -= alpha*result.vec
                ls_itercount += 1

                # Metadata update
                update_local_meta(local_meta, (self.iter_count, ls_itercount))

                # Just evaluate the model with the new points
                if self.options['solve_subsystems'] is True:
                    system.children_solve_nonlinear(local_meta)
                system.apply_nonlinear(params, unknowns, resids, local_meta)

                self.recorders.record_iteration(system, local_meta)

                f_norm = resids.norm()
                if self.options['iprint'] > 1:
                    self.print_norm('BK_TKG', system.pathname, ls_itercount, f_norm,
                                    f_norm0, indent=1, solver='LS')

            # Reset backtracking
            alpha = alpha_base

        # Need to make sure the whole workflow is executed at the final
        # point, not just evaluated.
        #self.iter_count += 1
        #update_local_meta(local_meta, (self.iter_count, 0))
        #system.children_solve_nonlinear(local_meta)

        if self.options['iprint'] > 0:

            if self.iter_count == maxiter or isnan(f_norm):
                msg = 'FAILED to converge after max iterations'
            else:
                msg = 'converged'

            self.print_norm('NEWTON', system.pathname, self.iter_count, f_norm,
                            f_norm0, msg=msg)



"""
OpenMDAO design-of-experiments driver implementing the Uniform method.
"""

from openmdao.drivers.predeterminedruns_driver import PredeterminedRunsDriver
from six import moves, iteritems
import numpy as np


class UniformDriver(PredeterminedRunsDriver):
    """Design-of-experiments Driver implementing the Uniform method.
    """

    def __init__(self, num_samples=1):
        super(UniformDriver, self).__init__()
        self.num_samples = num_samples

    def _build_runlist(self):
        """Build a runlist based on a uniform distribution."""

        for i in moves.xrange(self.num_samples):
            yield dict(((key, np.random.uniform(bound['low'], bound['high']))
                        for key, bound in iteritems(self.get_desvar_metadata())))



""" OpenMDAO LinearSolver that uses linear Gauss Seidel."""

from __future__ import print_function

from six import iteritems, itervalues
from collections import OrderedDict

from openmdao.solvers.solver_base import LinearSolver


class LinearGaussSeidel(LinearSolver):
    """ LinearSolver that uses linear Gauss Seidel.

    Options
    -------
    options['atol'] :  float(1e-12)
        Absolute convergence tolerance.
    options['iprint'] :  int(0)
        Set to 0 to disable printing, set to 1 to print the residual to stdout each iteration, set to 2 to print subiteration residuals as well.
    options['maxiter'] :  int(1)
        Maximum number of iterations.
    options['mode'] :  str('auto')
        Derivative calculation mode, set to 'fwd' for forward mode, 'rev' for reverse mode, or 'auto' to let OpenMDAO determine the best mode.
    options['rtol'] :  float(1e-10)
        Absolute convergence tolerance.

    """

    def __init__(self):
        super(LinearGaussSeidel, self).__init__()

        opt = self.options
        opt.add_option('atol', 1e-12,
                       desc='Absolute convergence tolerance.')
        opt.add_option('rtol', 1e-10,
                       desc='Absolute convergence tolerance.')
        opt.add_option('maxiter', 1,
                       desc='Maximum number of iterations.')
        opt.add_option('mode', 'auto', values=['fwd', 'rev', 'auto'],
                       desc="Derivative calculation mode, set to 'fwd' for " +
                       "forward mode, 'rev' for reverse mode, or 'auto' to " +
                       "let OpenMDAO determine the best mode.")
        opt.add_option('single_voi_relevance_reduction',
                        False, values=[True, False],
                        desc="If True, use relevance reduction even for"
                              " individual variables of interest. This "
                              "may increase performance but will use "
                              "more memory.")

    def solve(self, rhs_mat, system, mode):
        """ Solves the linear system for the problem in self.system. The
        full solution vector is returned.

        Args
        ----
        rhs_mat : dict of ndarray
            Dictionary containing one ndarry per top level quantity of
            interest. Each array contains the right-hand side for the linear
            solve.

        system : `System`
            Parent `System` object.

        mode : string
            Derivative mode, can be 'fwd' or 'rev'.

        Returns
        -------
        dict of ndarray : Solution vectors
        """

        dumat = system.dumat
        drmat = system.drmat
        dpmat = system.dpmat
        gs_outputs = system.gs_outputs
        relevance = system._relevance
        fwd = mode == 'fwd'

        system.clear_dparams()
        for names in system._relevance.vars_of_interest():
            for name in names:
                if name in dumat:
                    dumat[name].vec[:] = 0.0
        dumat[None].vec[:] = 0.0

        vois = rhs_mat.keys()
        # John starts with the following. It is not necessary, but
        # uncommenting it helps to debug when comparing print outputs to his.
        # for voi in vois:
        #    drmat[voi].vec[:] = -rhs_mat[voi]

        sol_buf = OrderedDict()

        f_norm0, f_norm = 1.0, 1.0
        self.iter_count = 0
        maxiter = self.options['maxiter']
        while self.iter_count < maxiter and f_norm > self.options['atol'] and f_norm/f_norm0 > self.options['rtol']:

            if fwd:

                for sub in itervalues(system._subsystems):

                    for voi in vois:
                        #print('pre scatter', sub.pathname, 'dp', dpmat[voi].vec,
                        #      'du', dumat[voi].vec, 'dr', drmat[voi].vec)
                        system._transfer_data(sub.name, deriv=True,
                                              var_of_interest=voi)
                        #print('pre apply', sub.pathname, 'dp', dpmat[voi].vec,
                        #      'du', dumat[voi].vec, 'dr', drmat[voi].vec)

                    # we need to loop over all subsystems in order to make
                    # the necessary collective calls to scatter, but only
                    # active subsystems do anything else
                    if not sub.is_active():
                        continue

                    # print(sub.name, sorted(gs_outputs['fwd'][sub.name][None]))

                    # Groups and all other systems just call their own
                    # apply_linear.
                    sub._sys_apply_linear(mode, ls_inputs=system._ls_inputs, vois=vois,
                                          gs_outputs=gs_outputs['fwd'][sub.name])

                    # for voi in vois:
                    #    print('post apply', dpmat[voi].vec, dumat[voi].vec, drmat[voi].vec)

                    for voi in vois:
                        drmat[voi].vec *= -1.0
                        drmat[voi].vec += rhs_mat[voi]
                        dpmat[voi].vec[:] = 0.0

                    sub.solve_linear(sub.dumat, sub.drmat, vois, mode=mode)

                    # for voi in vois:
                    #    print('post solve', dpmat[voi].vec, dumat[voi].vec, drmat[voi].vec)

                for voi in vois:
                    sol_buf[voi] = dumat[voi].vec

            else:

                for sub in reversed(list(itervalues(system._subsystems))):

                    active = sub.is_active()

                    for voi in vois:
                        if active:
                            dumat[voi].vec *= 0.0

                        #print('pre scatter', sub.pathname, voi, dpmat[voi].vec, dumat[voi].vec, drmat[voi].vec)
                        system._transfer_data(sub.name, mode='rev', deriv=True, var_of_interest=voi)
                        #print('post scatter', sub.pathname, voi, dpmat[voi].vec, dumat[voi].vec, drmat[voi].vec)

                        if active:
                            dumat[voi].vec *= -1.0
                            dumat[voi].vec += rhs_mat[voi]

                    # we need to loop over all subsystems in order to make
                    # the necessary collective calls to scatter, but only
                    # active subsystems do anything else
                    if not active:
                        continue

                    sub.solve_linear(sub.dumat, sub.drmat, vois, mode=mode)
                    #for voi in vois:
                        #print('post solve', dpmat[voi].vec, dumat[voi].vec, drmat[voi].vec)

                    #print(sub.name, sorted(gs_outputs['rev'][sub.name][None]))

                    # Groups and all other systems just call their own
                    # apply_linear.
                    sub._sys_apply_linear(mode, ls_inputs=system._ls_inputs, vois=vois,
                                         gs_outputs=gs_outputs['rev'][sub.name])

                    #for voi in vois:
                        #print('post apply', system.dpmat[voi].vec, dumat[voi].vec, drmat[voi].vec)

                for voi in vois:
                    sol_buf[voi] = drmat[voi].vec

            self.iter_count += 1
            if maxiter == 1:
                f_norm = 0.0
            else:
                f_norm = self._norm(system, mode, rhs_mat)

            if self.options['iprint'] > 0:
                self.print_norm('LN_GS', system.pathname, self.iter_count,
                                f_norm, f_norm0, indent=1, solver='LN')

        if self.options['iprint'] > 0:
            if maxiter > 1 and self.iter_count == maxiter:
                msg = 'FAILED to converge after max iterations'
            else:
                msg = 'converged'

            self.print_norm('LN_GS', system.pathname, self.iter_count, f_norm,
                            f_norm0, indent=1, solver='LN', msg=msg)

        return sol_buf

    def _norm(self, system, mode, rhs_mat):
        """ Computes the norm of the linear residual

        Args
        ----
        system : `System`
            Parent `System` object.

        mode : string
            Derivative mode, can be 'fwd' or 'rev'.

        rhs_mat : dict of ndarray
            Dictionary containing one ndarry per top level quantity of
            interest. Each array contains the right-hand side for the linear
            solve.
        """

        if mode == 'fwd':
            rhs_vec = system.drmat
        else:
            rhs_vec = system.dumat

        # we used to build gs_outputs up using dumat, but dumat is already
        # identical to gs_outputs in the vois we care about, so just use it.
        system._sys_apply_linear(mode, ls_inputs=system._ls_inputs, vois=rhs_mat.keys(),
                                gs_outputs=system.dumat)

        norm = 0.0
        for voi, rhs in iteritems(rhs_mat):
            rhs_vec[voi].vec[:] -= rhs
            norm += rhs_vec[voi].norm()**2

        return norm**0.5



import unittest
from six import itervalues

from openmdao.api import ExecComp, IndepVarComp, Problem, Group


class TestLinearGaussSeidel(unittest.TestCase):
    def setUp(self):
        self.p = Problem(Group())
        root = self.p.root

        root.add('P1', IndepVarComp('x', 2.0))
        root.add('P2', IndepVarComp('x', 2.0))

        root.add('C2', ExecComp('y = 2.0*x'))
        root.add('C3', ExecComp('y = 2.0*x'))
        root.add('C4', ExecComp('y = 2.0*x'))
        root.add('C5', ExecComp('y = 2.0*x'))
        root.add('C6', ExecComp('y = 2.0*x'))
        root.add('C7', ExecComp('y = 2.0*x'))
        root.add('C8', ExecComp('y = 2.0*x1 + 3.0*x2'))

        root.connect('P1.x', 'C2.x')
        root.connect('P1.x', 'C3.x')
        root.connect('P2.x', 'C4.x')
        root.connect('P2.x', 'C5.x')

        root.connect('C2.y', 'C8.x1')
        root.connect('C4.y', 'C8.x2')

        root.connect('C3.y', 'C6.x')
        root.connect('C5.y', 'C7.x')

    def test_relevant(self):
        p = self.p
        root = p.root

        p.driver.add_desvar('P1.x')
        p.driver.add_objective('C8.y')

        p.setup(check=False)

        rels = {
            'P1.x': ['P1.x', 'C2.x', 'C2.y', 'C8.x1', 'C8.y'],
            'C8.y': ['P1.x', 'C2.x', 'C2.y', 'C8.x1', 'C8.y']
        }

        not_rels = {
            'P1.x': ['P2.x', 'C4.x', 'C4.y', 'C5.x', 'C5.y', 'C7.x', 'C7.y', 'C8.x2'],
            'C8.y': ['P2.x', 'C4.x', 'C4.y', 'C5.x', 'C5.y', 'C7.x', 'C7.y', 'C8.x2']
        }

        for voi, voi_rels in rels.items():
            for voi_rel in voi_rels:
                self.assertTrue(root._relevance.is_relevant(voi, voi_rel),
                                msg="%s should be True" % voi_rel)

        for voi, voi_nrels in not_rels.items():
            for voi_nrel in voi_nrels:
                self.assertFalse(root._relevance.is_relevant(voi, voi_nrel),
                                 msg="%s should be False" % voi_nrel)

    def test_relevant_systems(self):
        p = self.p
        root = p.root

        p.driver.add_desvar('P1.x')
        p.driver.add_objective('C8.y')

        p.setup(check=False)

        rel_systems = ['P1', 'C2', 'C8']
        for s in itervalues(root._subsystems):
            if s.pathname in rel_systems:
                self.assertTrue(root._relevance.is_relevant_system('P1.x', s),
                                msg="%s should be relevant" % s.pathname)
                self.assertTrue(root._relevance.is_relevant_system('C8.y', s),
                                msg="%s should be relevant" % s.pathname)
            else:
                self.assertFalse(root._relevance.is_relevant_system('P1.x', s),
                                 msg="%s should be irrelevant" % s.pathname)
                self.assertFalse(root._relevance.is_relevant_system('C8.y', s),
                                 msg="%s should be irrelevant" % s.pathname)



"""
Who is Testing the Testers?
"""

import logging
import os.path
import sys

import unittest
import numpy as np

from openmdao.api import Group, Problem, IndepVarComp
from openmdao.test.util import assert_rel_error, problem_derivatives_check
from openmdao.test.simple_comps import SimpleCompWrongDeriv


class TestCase(unittest.TestCase):
    """ Test Test functions. """

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_rel_error_inf_nan(self):

        try:
            assert_rel_error(self, float('nan'), 6.5, 0.0001)
        except AssertionError as exc:
            msg = "actual nan, desired 6.5, rel error nan, tolerance 0.0001"
            self.assertEqual(str(exc), msg)
        else:
            self.fail('Expected AssertionError')

        try:
            assert_rel_error(self, float('inf'), 6.5, 0.0001)
        except AssertionError as exc:
            msg = "actual inf, desired 6.5, rel error inf, tolerance 0.0001"
            self.assertEqual(str(exc), msg)
        else:
            self.fail('Expected AssertionError')

        # We may actually want this to work for some tests.
        assert_rel_error(self, float('nan'), float('nan'), 0.0001)

    def test_rel_error_array(self):

        try:
            assert_rel_error(self, 1e-2*np.ones(3), np.zeros(3), 1e-3)
        except AssertionError as exc:
            msg="arrays do not match, rel error 1.732e-02 > tol (1.000e-03)"
            self.assertEqual(msg, str(exc))
        else:
            self.fail("Expected Assertion Error")

        err = assert_rel_error(self, 1e-2*np.ones(3), 1e-2*np.ones(3), 1e-10)
        self.assertEqual(err, 0.0)

        err = assert_rel_error(self, 1e-2*np.ones(3), 1.00001e-2*np.ones(3), 1e-3)
        self.assertLessEqual(err, 1e-5)


    def test_problem_deriv_test(self):

        prob = Problem()
        prob.root = Group()
        prob.root.add('comp', SimpleCompWrongDeriv())
        prob.root.add('p1', IndepVarComp('x', 2.0))
        prob.root.connect('p1.x', 'comp.x')

        prob.setup(check=False)
        prob.run()

        data = prob.check_partial_derivatives(out_stream=None)

        try:
            problem_derivatives_check(self, prob)
        except AssertionError as err:
            self.assertIn("not less than or equal to 1e-05", err.args[0])



if __name__ == "__main__":
    unittest.main()



""" Formerly the one-two-one-two-one test. A model that diverges, converges,
diverges, then converges again. """

from openmdao.components.indep_var_comp import IndepVarComp
from openmdao.core.group import Group
from openmdao.core.parallel_group import ParallelGroup
from openmdao.core.component import Component


class Comp1(Component):

    def __init__(self):
        super(Comp1, self).__init__()
        self.add_param('x1', 1.0)
        self.add_output('y1', 1.0)
        self.add_output('y2', 1.0)

    def solve_nonlinear(self, params, unknowns, resids):
        """ Runs the component."""
        unknowns['y1'] = 2.0*params['x1']**2
        unknowns['y2'] = 3.0*params['x1']

    def linearize(self, params, unknowns, resids):
        """Returns the Jacobian."""
        J = {}
        J[('y1', 'x1')] = 4.0*params['x1']
        J[('y2', 'x1')] = 3.0
        return J

class Comp2(Component):

    def __init__(self):
        super(Comp2, self).__init__()
        self.add_param('x1', 1.0)
        self.add_output('y1', 1.0)

    def solve_nonlinear(self, params, unknowns, resids):
        """ Runs the component."""
        unknowns['y1'] = 0.5*params['x1']

    def linearize(self, params, unknowns, resids):
        """Returns the Jacobian."""
        J = {}
        J[('y1', 'x1')] = 0.5
        return J

class Comp3(Component):

    def __init__(self):
        super(Comp3, self).__init__()
        self.add_param('x1', 1.0)
        self.add_output('y1', 1.0)

    def solve_nonlinear(self, params, unknowns, resids):
        """ Runs the component."""
        unknowns['y1'] = 3.5*params['x1']

    def linearize(self, params, unknowns, resids):
        """Returns the Jacobian."""
        J = {}
        J[('y1', 'x1')] = 3.5
        return J

class Comp4(Component):

    def __init__(self):
        super(Comp4, self).__init__()
        self.add_param('x1', 1.0)
        self.add_param('x2', 1.0)
        self.add_output('y1', 1.0)
        self.add_output('y2', 1.0)

    def solve_nonlinear(self, params, unknowns, resids):
        """ Runs the component."""
        unknowns['y1'] = params['x1'] + 2.0*params['x2']
        unknowns['y2'] = 3.0*params['x1'] - 5.0*params['x2']

    def linearize(self, params, unknowns, resids):
        """Returns the Jacobian."""
        J = {}
        J[('y1', 'x1')] = 1.0
        J[('y1', 'x2')] = 2.0
        J[('y2', 'x1')] = 3.0
        J[('y2', 'x2')] = -5.0
        return J

class Comp5(Component):

    def __init__(self):
        super(Comp5, self).__init__()
        self.add_param('x1', 1.0)
        self.add_output('y1', 1.0)

    def solve_nonlinear(self, params, unknowns, resids):
        """ Runs the component."""
        unknowns['y1'] = 0.8*params['x1']

    def linearize(self, params, unknowns, resids):
        """Returns the Jacobian."""
        J = {}
        J[('y1', 'x1')] = 0.8
        return J

class Comp6(Component):

    def __init__(self):
        super(Comp6, self).__init__()
        self.add_param('x1', 1.0)
        self.add_output('y1', 1.0)

    def solve_nonlinear(self, params, unknowns, resids):
        """ Runs the component."""
        unknowns['y1'] = 0.5*params['x1']

    def linearize(self, params, unknowns, resids):
        """Returns the Jacobian."""
        J = {}
        J[('y1', 'x1')] = 0.5
        return J

class Comp7(Component):

    def __init__(self):
        super(Comp7, self).__init__()
        self.add_param('x1', 1.0)
        self.add_param('x2', 1.0)
        self.add_output('y1', 1.0)

    def solve_nonlinear(self, params, unknowns, resids):
        """ Runs the component."""
        unknowns['y1'] = params['x1'] + 3.0*params['x2']

    def linearize(self, params, unknowns, resids):
        """Returns the Jacobian."""
        J = {}
        J[('y1', 'x1')] = 1.0
        J[('y1', 'x2')] = 3.0
        return J


class ConvergeDiverge(Group):
    """ Topology one - two - one - two - one. This model was critical in
    testing parallel reverse scatters."""

    def __init__(self):
        super(ConvergeDiverge, self).__init__()

        self.add('p', IndepVarComp('x', 2.0))

        self.add('comp1', Comp1())
        self.add('comp2', Comp2())
        self.add('comp3', Comp3())
        self.add('comp4', Comp4())
        self.add('comp5', Comp5())
        self.add('comp6', Comp6())
        self.add('comp7', Comp7())

        self.connect("p.x", "comp1.x1")
        self.connect('comp1.y1', 'comp2.x1')
        self.connect('comp1.y2', 'comp3.x1')
        self.connect('comp2.y1', 'comp4.x1')
        self.connect('comp3.y1', 'comp4.x2')
        self.connect('comp4.y1', 'comp5.x1')
        self.connect('comp4.y2', 'comp6.x1')
        self.connect('comp5.y1', 'comp7.x1')
        self.connect('comp6.y1', 'comp7.x2')


class ConvergeDivergePar(Group):
    """ Topology one - two - one - two - one. This model was critical in
    testing parallel reverse scatters."""

    def __init__(self):
        super(ConvergeDivergePar, self).__init__()

        self.add('p', IndepVarComp('x', 2.0))

        self.add('comp1', Comp1())
        par1 = self.add('par1', ParallelGroup())
        par1.add('comp2', Comp2())
        par1.add('comp3', Comp3())
        self.add('comp4', Comp4())
        par2 = self.add('par2', ParallelGroup())
        par2.add('comp5', Comp5())
        par2.add('comp6', Comp6())
        self.add('comp7', Comp7())

        self.connect("p.x", "comp1.x1")
        self.connect('comp1.y1', 'par1.comp2.x1')
        self.connect('comp1.y2', 'par1.comp3.x1')
        self.connect('par1.comp2.y1', 'comp4.x1')
        self.connect('par1.comp3.y1', 'comp4.x2')
        self.connect('comp4.y1', 'par2.comp5.x1')
        self.connect('comp4.y2', 'par2.comp6.x1')
        self.connect('par2.comp5.y1', 'comp7.x1')
        self.connect('par2.comp6.y1', 'comp7.x2')


class ConvergeDivergeGroups(Group):
    """ Topology one - two - one - two - one. This model was critical in
    testing parallel reverse scatters."""

    def __init__(self):
        super(ConvergeDivergeGroups, self).__init__()


        self.add('p', IndepVarComp('x', 2.0))

        sub1 = self.add('sub1', Group())
        sub1.add('comp1', Comp1())

        sub2 = sub1.add('sub2', Group())
        sub2.add('comp2', Comp2())
        sub2.add('comp3', Comp3())
        sub1.add('comp4', Comp4())

        sub3 = self.add('sub3', Group())
        sub3.add('comp5', Comp5())
        sub3.add('comp6', Comp6())
        self.add('comp7', Comp7())

        self.connect("p.x", "sub1.comp1.x1")
        self.connect('sub1.comp1.y1', 'sub1.sub2.comp2.x1')
        self.connect('sub1.comp1.y2', 'sub1.sub2.comp3.x1')
        self.connect('sub1.sub2.comp2.y1', 'sub1.comp4.x1')
        self.connect('sub1.sub2.comp3.y1', 'sub1.comp4.x2')
        self.connect('sub1.comp4.y1', 'sub3.comp5.x1')
        self.connect('sub1.comp4.y2', 'sub3.comp6.x1')
        self.connect('sub3.comp5.y1', 'comp7.x1')
        self.connect('sub3.comp6.y1', 'comp7.x2')


class SingleDiamond(Group):
    """ Topology one - two - one."""

    def __init__(self):
        super(SingleDiamond, self).__init__()

        self.add('p', IndepVarComp('x', 2.0))

        self.add('comp1', Comp1())
        self.add('comp2', Comp2())
        self.add('comp3', Comp3())
        self.add('comp4', Comp4())

        self.connect("p.x", "comp1.x1")
        self.connect('comp1.y1', 'comp2.x1')
        self.connect('comp1.y2', 'comp3.x1')
        self.connect('comp2.y1', 'comp4.x1')
        self.connect('comp3.y1', 'comp4.x2')


class SingleDiamondPar(Group):
    """ Topology one - two - one."""

    def __init__(self):
        super(SingleDiamondPar, self).__init__()

        self.add('p', IndepVarComp('x', 2.0))

        self.add('comp1', Comp1())
        sub = self.add('sub', ParallelGroup())
        sub.add('comp2', Comp2())
        sub.add('comp3', Comp3())
        self.add('comp4', Comp4())

        self.connect("p.x", "comp1.x1")
        self.connect('comp1.y1', 'sub.comp2.x1')
        self.connect('comp1.y2', 'sub.comp3.x1')
        self.connect('sub.comp2.y1', 'comp4.x1')
        self.connect('sub.comp3.y1', 'comp4.x2')


class SingleDiamondGrouped(Group):
    """ Topology one - two - one."""

    def __init__(self):
        super(SingleDiamondGrouped, self).__init__()

        self.add('p', IndepVarComp('x', 2.0))

        sub1 = self.add('sub1', Group())
        sub1.add('comp1', Comp1())
        sub1.add('comp2', Comp2())
        sub1.add('comp3', Comp3())
        self.add('comp4', Comp4())

        self.connect("p.x", "sub1.comp1.x1")
        self.connect('sub1.comp1.y1', 'sub1.comp2.x1')
        self.connect('sub1.comp1.y2', 'sub1.comp3.x1')
        self.connect('sub1.comp2.y1', 'comp4.x1')
        self.connect('sub1.comp3.y1', 'comp4.x2')
